﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class PayExtension
    {
        /// <summary>
        /// complate!
        /// 使用到账进行收款登记
        /// </summary>
        /// <param name="settlements">要被收款登记的结算列表</param>
        /// <param name="payment">用于收款登记的到账</param>
        /// <param name="accountPayee">收入账户</param>
        /// <param name="pays">生成的pay清单,Biz记录结算信息</param>
        /// <param name="accountDetail">生成的账户明细清单</param>
        /// <param name="amount">指定的费用,若不指定金额则默认全部</param>
        /// <returns>生成用于收款登记的到账子项</returns>
        public static Payment Paying(this List<Settlement> settlements, Payment payment, Account accountPayee, out List<Pay> pays, out AccountDetail accountDetail, decimal amount = 0, string remark = null)
        {
            settlements.ThrowIfNullOrEmpty("未传入结算信息!");
            payment.ThrowIfNull("未传入到账信息!");
            accountPayee.ThrowIfNull("未传入指定收款账户!");
            (settlements.Exists(o => o.Currency != payment.Currency || o.Currency != accountPayee.Currency)).ThrowIfTrue("存在不相符的货币单位!");
            if (payment.Type != PaymentType.Split && payment.Type != PaymentType.Pos && payment.Type != PaymentType.Cash && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("请选择正确的付款方式!");
            var amountSettlement = settlements.Sum(o => o.Amount - o.AmountPayment);
            if (amountSettlement <= 0) throw new Exception("当前结算已无需付款登记!");
            if (amount <= 0 || amount > amountSettlement) amount = amountSettlement;
            (payment.Amount - payment.AmountUsed < amount).ThrowIfTrue($"付款编号[{payment.SerialNumber}]余额不足!可用余额:{ payment.Amount - payment.AmountUsed}");

            var dynamicPayKvid = Guid.NewGuid();
            #region 创建入账Payment
            var rtns = new Payment();
            rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            rtns.SerialNumber = payment.SerialNumber;
            rtns.Kvid = dynamicPayKvid;
            rtns.RootKvid = payment.RootKvid;
            rtns.ParentKvid = payment.Kvid;
            rtns.Type = PaymentType.DynamicPay;
            rtns.BizId = accountPayee.Name;
            rtns.BizKvid = accountPayee.Kvid;
            rtns.BizType = typeof(Account).FullName;
            rtns.PayerName = payment.PayerName;
            rtns.PayerKvid = payment.PayerKvid;
            rtns.PayeeName = accountPayee.Name;
            rtns.PayeeKvid = accountPayee.Kvid;
            rtns.PayeeAccountName = accountPayee.Name;
            rtns.PayeeAccountKvid = accountPayee.Kvid;
            rtns.PayeeAccountOwnerName = accountPayee.OwnerName;
            rtns.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            rtns.Amount = amount;
            rtns.AmountSplited = 0;
            rtns.AmountUsed = amount;
            rtns.Summary = "收款登记";
            rtns.OperateTime = DateTime.Now;
            rtns.OperatorName = KiviiContext.CurrentMember.FullName;
            rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            rtns.Currency = payment.Currency;
            #endregion

            #region 收款账户变动
            accountDetail = new AccountDetail();
            accountDetail.AccountKvid = accountPayee.Kvid;
            accountDetail.BizId = payment.SerialNumber;
            accountDetail.BizKvid = payment.Kvid;
            accountDetail.BizType = typeof(Payment).FullName;
            accountDetail.PayerKvid = payment.PayerKvid;
            accountDetail.PayerName = payment.PayerName;
            accountDetail.PayeeKvid = payment.PayeeKvid;
            accountDetail.PayeeName = payment.PayeeName;
            accountDetail.PayeeAccountKvid = accountPayee.Kvid;
            accountDetail.PayeeAccountName = accountPayee.Name;
            accountDetail.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            accountDetail.AmountPayment = amount;
            accountDetail.Summary = "收款登记";
            accountDetail.Remark = remark;
            #endregion

            pays = new List<Pay>();
            foreach (var settlement in settlements)
            {
                if (settlement.AmountPayment >= settlement.Amount) continue;
                if (amount <= 0) continue;
                var currentAmount = (amount <= (settlement.Amount - settlement.AmountPayment) ? amount : (settlement.Amount - settlement.AmountPayment));

                var pay = new Pay();
                pay.BizId = settlement.SerialNumber;
                pay.BizKvid = settlement.Kvid;
                pay.BizType = typeof(Settlement).FullName; 
                pay.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                pay.OwnerName = KiviiContext.CurrentMember.DepartmentName;
                pay.PayerKvid = rtns.PayerKvid;
                pay.PayerName = rtns.PayerName;
                pay.FromKvid = rtns.Kvid;
                pay.FromType = PayType.Payment;
                pay.PayeeKvid = accountPayee.Kvid;
                pay.PayeeName = accountPayee.Name;
                pay.ToKvid = accountPayee.Kvid;
                pay.ToType = PayType.Account;
                pay.Type = "DynamicPay";
                pay.PayedTime = DateTime.Now;
                pay.Amount = currentAmount;
                pay.AmountPayment = currentAmount;
                pay.AmountDiscount = 0;
                pay.Currency = rtns.Currency;
                pay.OperateTime = DateTime.Now;
                pay.OperatorName = KiviiContext.CurrentMember.FullName;
                pay.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                pay.Remark = remark;
                pays.Add(pay);

                settlement.AmountPayment += currentAmount;
                settlement.AddOnlyProperties(o => o.AmountPayment);
                amount -= currentAmount;
            }

            return rtns;
        }

        /// <summary>
        /// complate!
        /// 使用账户进行收款登记
        /// </summary>
        /// <param name="settlements">要被收款登记的结算列表</param>
        /// <param name="accountPayer">用于收款登记的账户</param>
        /// <param name="accountPayee">收入账户</param>
        /// <param name="pays">生成的pay清单,Biz记录结算信息</param>
        /// <param name="amount">指定的费用,若不指定金额则默认全部</param>
        /// <returns>生成的账户明细清单</returns>
        public static List<AccountDetail> Paying(this List<Settlement> settlements, Account accountPayer, Account accountPayee, out List<Pay> pays, decimal amount = 0, string remark = null)
        {
            settlements.ThrowIfNullOrEmpty("未传入结算信息!");
            accountPayer.ThrowIfNull("未传入指定付款账户!");
            accountPayee.ThrowIfNull("未传入指定收款账户!");
            (settlements.Exists(o => o.Currency != accountPayer.Currency || o.Currency != accountPayee.Currency)).ThrowIfTrue("存在不相符的货币单位!");
            (accountPayer.Type != AccountType.Deposit && accountPayer.Type != AccountType.Discount).ThrowIfTrue("请选择正确的付款账户!");
            var amountSettlement = settlements.Sum(o => o.Amount - o.AmountPayment);
            if (amountSettlement <= 0) throw new Exception("当前结算已无需付款登记!");
            if (amount <= 0 || amount > amountSettlement) amount = amountSettlement;
            (accountPayer.Amount + accountPayer.CreditLine < amount).ThrowIfTrue($"账户[{accountPayer.Name}]余额不足!可用余额:{accountPayer.Amount + accountPayer.CreditLine}");

            var rtns = new List<AccountDetail>();
            #region 收款账户变动
            var payeeDetail = new AccountDetail();
            payeeDetail.AccountKvid = accountPayee.Kvid;
            payeeDetail.BizId = accountPayee.SerialNumber;
            payeeDetail.BizKvid = accountPayee.Kvid;
            payeeDetail.BizType = typeof(Account).FullName;
            payeeDetail.PayerKvid = accountPayer.Kvid;
            payeeDetail.PayerName = accountPayer.Name;
            payeeDetail.PayerAccountKvid = accountPayer.Kvid;
            payeeDetail.PayerAccountName = accountPayer.Name;
            payeeDetail.PayerAccountSerialNumber = accountPayer.SerialNumber;
            payeeDetail.PayeeKvid = accountPayee.Kvid;
            payeeDetail.PayeeName = accountPayee.Name;
            payeeDetail.PayeeAccountKvid = accountPayee.Kvid;
            payeeDetail.PayeeAccountName = accountPayee.Name;
            payeeDetail.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            payeeDetail.AmountPayment = amount;
            payeeDetail.Summary = "收款登记";
            payeeDetail.Remark = remark;
            rtns.Add(payeeDetail);

            var payerDetail = new AccountDetail();
            payerDetail.AccountKvid = accountPayer.Kvid;
            payerDetail.BizId = accountPayee.SerialNumber;
            payerDetail.BizKvid = accountPayee.Kvid;
            payerDetail.BizType = typeof(Account).FullName;
            payerDetail.PayerKvid = accountPayer.Kvid;
            payerDetail.PayerName = accountPayer.Name;
            payerDetail.PayerAccountKvid = accountPayer.Kvid;
            payerDetail.PayerAccountName = accountPayer.Name;
            payerDetail.PayerAccountSerialNumber = accountPayer.SerialNumber;
            payerDetail.PayeeKvid = accountPayee.Kvid;
            payerDetail.PayeeName = accountPayee.Name;
            payerDetail.PayeeAccountKvid = accountPayee.Kvid;
            payerDetail.PayeeAccountName = accountPayee.Name;
            payerDetail.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            payerDetail.AmountPayment = amount;
            payerDetail.Summary = "收款登记";
            payerDetail.Remark = remark;
            rtns.Add(payerDetail);
            #endregion

            pays = new List<Pay>();
            foreach (var settlement in settlements)
            {
                if (settlement.AmountPayment >= settlement.Amount) continue;
                if (amount <= 0) continue;
                var currentAmount = amount <= (settlement.Amount - settlement.AmountPayment) ? amount : (settlement.Amount - settlement.AmountPayment);

                var pay = new Pay();
                pay.BizId = settlement.SerialNumber;
                pay.BizKvid = settlement.Kvid;
                pay.BizType = typeof(Settlement).FullName;
                pay.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                pay.OwnerName = KiviiContext.CurrentMember.DepartmentName;
                pay.PayerKvid = accountPayer.Kvid;
                pay.PayerName = accountPayer.Name;
                pay.FromKvid = accountPayer.Kvid;
                pay.FromType = PayType.Account;
                pay.PayeeKvid = accountPayee.Kvid;
                pay.PayeeName = accountPayee.Name;
                pay.ToKvid = accountPayee.Kvid;
                pay.ToType = PayType.Account;
                pay.Type = "DynamicPay";
                pay.PayedTime = DateTime.Now;
                pay.Amount = currentAmount;
                if (accountPayer.Type == AccountType.Discount)//要是 折扣账户 则记录到AmountDiscount
                {
                    pay.AmountPayment = 0;
                    pay.AmountDiscount = currentAmount;
                }
                else
                {
                    pay.AmountPayment = currentAmount;
                    pay.AmountDiscount = 0;
                }
                pay.Currency = accountPayer.Currency;
                pay.OperateTime = DateTime.Now;
                pay.OperatorName = KiviiContext.CurrentMember.FullName;
                pay.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                pay.Remark = remark;
                pays.Add(pay);

                settlement.AmountPayment += currentAmount;
                settlement.AddOnlyProperties(o => o.AmountPayment);
                amount -= currentAmount;
            }

            return rtns;
        }

        /// <summary>
        /// complate!
        /// 作废
        /// </summary>
        /// <param name="pay"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Pay Offset(this Pay pay, string remark)
        {
            pay.ThrowIfNull("付款记录不能为空!");
            (pay.OffsetKvid != Guid.Empty).ThrowIfTrue("收项已冲账");
            (pay.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");

            #region pay冲账
            var newPayKvid = Guid.NewGuid();
            pay.OffsetKvid = newPayKvid;
            pay.AddOnlyProperties(o => o.OffsetKvid);
            pay.Remark = $"{remark}";
            pay.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetPay = new Pay();
            offsetPay.PopulateWith(pay);
            offsetPay.OffsetKvid = pay.Kvid;//新的冲帐关联用原来的Kvid
            offsetPay.Kvid = newPayKvid;
            offsetPay.SerialNumber = pay.SerialNumber.IsNullOrEmpty() ? "" : ("-" + pay.SerialNumber);
            offsetPay.Remark = $"{remark}";
            offsetPay.AmountDiscount = 0 - offsetPay.AmountDiscount;
            offsetPay.AmountPayment = 0 - offsetPay.AmountPayment;
            offsetPay.Amount = 0 - offsetPay.Amount;//反向值
            offsetPay.OperatorName = KiviiContext.CurrentMember.FullName;
            offsetPay.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            #endregion

            return offsetPay;
        }

        ///// <summary>
        ///// 付款登记折扣金额记录
        ///// </summary>
        ///// <param name="pay"></param>
        ///// <param name="discountAccount"></param>
        ///// <param name="amount"></param>
        ///// <param name="remark"></param>
        ///// <returns></returns>
        //public static Discount Discount(this Pay pay, Account discountAccount, decimal amount, string remark)
        //{
        //    pay.ThrowIfNull("未传入付款登记信息!");
        //    discountAccount.ThrowIfNull("请传入收款账户!");
        //    (discountAccount.Type != AccountType.Discount).ThrowIfTrue("请选择正确的折扣账户!");
        //    (discountAccount.Currency != pay.Currency).ThrowIfTrue("存在不相符的货币单位!");
        //    (amount <= 0).ThrowIfTrue("抹零抵扣金额不能少于0!");
        //    (amount > pay.AmountSettlement - pay.AmountPayment).ThrowIfTrue("折扣金额超过付款登记金额!");

        //    pay.AmountPayment += amount;
        //    pay.AmountDiscount = amount;
        //    pay.Amount = pay.AmountPayment - pay.AmountDiscount;
        //    pay.Remark = remark;

        //    #region 记录折扣抹零操作
        //    var rtns = new Discount();
        //    rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
        //    rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
        //    rtns.PayKvid = pay.Kvid;
        //    rtns.AccountKvid = discountAccount.Kvid;
        //    rtns.AccountName = discountAccount.Name;
        //    rtns.AccountSerialNumber = discountAccount.SerialNumber;
        //    rtns.Amount = amount;
        //    rtns.Currency = pay.Currency;
        //    rtns.OperateTime = DateTime.Now;
        //    rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
        //    rtns.OperatorName = KiviiContext.CurrentMember.FullName;
        //    rtns.Summary = $"{rtns.OperatorName}于{rtns.OperateTime} 对{pay.PayerName}[{pay.SerialNumber}]进行抹零折扣操作,金额为:{amount} {pay.Currency}";
        //    rtns.Remark = remark;
        //    #endregion

        //    return rtns;
        //}
    }
}
